// Funciones de Ratatui

/**	FunciÃ³n auxiliar para encontrar elementos del documento por su identificador (id).
 	@param id El identificador del elemento a buscar.
**/
function $(id){
	return document.getElementById(id);
}

///	@class Remy Clase Remy con la funcionalidad para las pruebas.
var Remy = {							// Es una "clase estática".
	debugar: false,
	divResultados: 'divResultados',				/// Identificador del Div en el que se mostrarÃ¡n los resultados de los test.
	divTest: 'divTest',					/// Identificador del Div en el que se mostrarÃ¡ el cÃ³digo del test.
	iframeTest: 'iframeTest',				/// Identificador del iframe en el que se cargarÃ¡n los tests para parsearlos.
	tiempoEspera: 2,					/// NÃºmero de segundos de espera al cargar tests y pÃ¡ginas.
	test: Array(),						/// Array que contendrÃ¡ las lÃ­neas del test.
	resultado: Array(),					/// Array con el resultado de cada lÃ­nea de test.
	//resultado: true,					/// Booleano indicando si hubo fallos en la ejecuciÃ³n.
	iframeTestCargado: false,				/// Ãndica si estÃ¡ cargada el iframe.
	iframeCargado: false,					/// Indica si estÃ¡ cargada el iframe.
	esperarACargar: 0,
	indice: null,						/// Ãndice de la lÃ­nea de test en ejecuciÃ³n.
	pararEjecucion: false,

	/**	Carga un fichero con tests, pero no continua la ejecuciÃ³n.
	 	Los tests se cargan en un iframe oculto para parsearlos y pasarlos al fieldset de tests.
		@param URL La URL del fichero a cargar
	**/
	cargarTestInicial: function(URL){
		Remy.debug("En cargarTestInicial("+URL+")");
		Remy.test = new Array();
		Remy.resultado = new Array();
		Remy._cargarTest(URL, false);
		return null;		//No podemos saber si lo ha hecho bien.
	},
	
	/**	Muestra un mensaje de depuraciÃ³n
	 **/
	debug: function(mensaje){
		if (Remy.debugar) Remy.mensaje(mensaje);
	},
	
	/**	Escribe un mensaje en la salida.
		@param mensaje Texto del mensaje a escribir.
	 	@param estado Uno de los siguientes textos: correcto, fallo o actual
	**/
	mensaje: function(mensaje, estado){
		var texto = document.createTextNode(mensaje);
		var div = document.createElement('div');
		if (estado) div.setAttribute("class", estado);
		div.appendChild(texto);
		$(Remy.divResultados).appendChild(div);
		return true;
	},

	/**	(Privada) Carga un fichero con tests.
	 	Los tests se cargan en un iframe oculto para parsearlos y pasarlos al fieldset de tests.
		@param URL La URL del fichero a cargar
		@param ejecutar Booleano indicando si debe continuarse con la ejecuciÃ³n tras cargar el test.
	**/
	_cargarTest: function(URL, continuar){
		Remy.debug("En _cargarTest("+URL+","+continuar+")");
		Remy.iframeTestCargado = false;
		$(Remy.iframeTest).src = URL;
		Remy._esperarCargaTest(continuar);
		return null;		//No podemos saber si lo ha hecho bien.
	},

	/**	Espera la carga del iframe de test y lo parsea.
		@param continuar Booleano indicando si tras la carga del test debe continuarse con la ejecuciÃ³n del test.
	**/
	_esperarCargaTest: function(continuar){
		if(!Remy.iframeTestCargado){
			Remy.pararEjecucion = true;	//Para que no siga ejecutando y espere...
			Remy.debug("Esperando la carga del test...");
			window.setTimeout('Remy._esperarCargaTest('+continuar+')', Remy.tiempoEspera*1000);
		}
		else{
			Remy.debug("Â¡Test Cargado!");
			Remy.parsear();
			if (continuar) {
				Remy.debug("Continuando con la ejecuciÃ³n del test");
				Remy.ejecutar();
			}
		}
	},

	/**	Prepara el código de test para que pueda ser ejecutado.
		Lo lee del iframeTest y lo aÃ±ade a la pila de test.
	**/
	parsear: function(){
		var codigo = $(Remy.iframeTest).contentDocument.getElementsByTagName("pre")[0].firstChild.nodeValue;	//Vale, parece raro y complicado, pero ahí­ está el código cargado (ver DOM Inspector).
		var test = codigo.split("\n");		// Separamos las líneas por retornos de carro.
		if (Remy.indice == null)		// Se trata de la carga inicial del test. No está en ejecución
			indice = 0;
		else{					// Es una carga con el test en ejecuciÃ³n.
			indice = Remy.indice-1;
			Remy.test.splice(indice, 1);	// Se elimina la lÃ­nea de carga de test para poder reiniciar (si lo pide el usuario).
		}
		for (var i=0; i<test.length; i++)	// Se insertan las lÃ­neas en la pila.
			Remy.test.splice(indice++, 0, test[i]);
		Remy.debug("Fin del parseo");
		Remy.mostrarTest();
	},

	/**	Muestra el test cargado en el Ã¡rea de test
	**/
	mostrarTest: function(){
		Remy.debug("En mostrarTest");
		Remy.debug("Limpiando el Ã¡rea de test");
		divTest = $(Remy.divTest);
		while (divTest.hasChildNodes()) divTest.removeChild(divTest.firstChild);
		Remy.debug("Mostrando el test");
		for (var i=0; i< Remy.test.length; i++){
			var texto = document.createTextNode(Remy.test[i]);
			var div = document.createElement('div');
			if (Remy.resultado[i]) div.setAttribute("class", Remy.resultado[i]);
			div.appendChild(texto);
			divTest.appendChild(div);
		}
	},

	/**	Inicia la ejecuciÃ³n del test cargado
	 **/
	iniciar: function(){
		Remy.indice = 0;
		Remy.ejecutar();
	},

	/**	@fn ejecutar FunciÃ³n principal que ejecuta el script de pruebas.
		El script se ejecuta desde el punto en el que se detuvo la Ãºltima vez (ver esperar).
	**/
	ejecutar: function (){
		Remy.debug("En ejecutar");
		Remy.pararEjecucion = false;
		while ((Remy.indice < Remy.test.length) && !Remy.pararEjecucion){
			Remy.marcarTest('actual');
			resultado = null;
			if (Remy.test[Remy.indice].match(/(^\s*$)|(^\/\/)/) == null){	// No es un comentario
				Remy.debug("Ejecutando "+Remy.test[Remy.indice]);
				resultado = eval("Remy."+Remy.test[Remy.indice]);
			}
			if (resultado === null) Remy.marcarTest('ejecutado');
			if (resultado === true) Remy.marcarTest('correcto');
			if (resultado === false) Remy.marcarTest('fallo');
			Remy.indice++;
		}
	},

	/**	Marca el test actual en el Ã¡rea de test
	 	@param estado Uno de los siguientes textos: correcto, fallo o actual
	**/
	marcarTest: function(estado){
		Remy.resultado[Remy.indice] = estado;
		if(!$(Remy.divTest).hasChildNodes()) return;
		$(Remy.divTest).childNodes.item(Remy.indice).setAttribute('class',estado);	//Marcamos el test actual;
	},

	/**	Carga un fichero con tests.
	 	Los tests se cargan en un iframe oculto para parsearlos y pasarlos al fieldset de tests.
		@param URL La URL del fichero a cargar
	**/
	cargarTest: function(URL){
		Remy._cargarTest(URL, true);
		return null;		//No podemos saber si lo ha hecho bien.
	},

	/**	Carga el fichero JS que indique el usuario.
		@param fichero Path al fichero JS.
	**/
	cargarJS: function (fichero){
		Remy.debug("CargandoJS("+fichero+")");
		var head = document.getElementsByTagName('head')[0];
		var script = document.createElement('script');
		script.src = fichero;
		script.type = 'text/javascript';
		head.appendChild(script);
		Remy.esperar();
		return null;		//No podemos saber si lo ha hecho bien.
	},

	/**	Carga una URL en la iframe de Ratatui.
		@param URL La URL de la ventana a cargar.
	**/
	cargar: function(URL){
		Remy.debug("Cargando("+URL+")");
		Remy.iframeCargado = false;
		$('iframe').src = URL;
		Remy._esperarCarga();
		return null;		//No podemos saber si lo ha hecho bien.
	},

	verPagina: function(){
	/**-	Devuelve el cÃ³digo fuente de la pÃ¡gina cargada.
	**/
		var html = $('iframe').contentDocument.getElementsByTagName("html")[0].innerHTML;
		//html = html.replace(/\"/g,'#');
		//html = html.replace(/\n/g,'');
		return html;
	},

	verPaginaMD5: function(){
	/**-	Devuelve el cÃ³digo fuente de la pÃ¡gina cargada codificado en MD5.
		Al cÃ³digo fuente le sustituye las comillas dobles y eliminamos los retornos de carro.
		De esta forma, el cÃ³digo puede copiarse y pegarse para compararse con el resultado de la funciÃ³n cargar.
	**/
		var html = $('iframe').contentDocument.getElementsByTagName("html")[0].innerHTML;
		return Remy.md5(html);
	},

	/**	Comprueba que dos valores son iguales y escribe el resultado como test.
		@param val1 Primer valor a comparar.
		@param val2 Segundo valor a comparar.
		@param	mensaje Texto que se escribirÃ¡ en la salida del test.
	**/
	probarIgual: function(val1, val2, mensaje){
		resultado = false;
		if (val1 == val2) resultado = true;
		if (!resultado){
			Remy.resultado = false;
			Remy.marcarTest('fallo');
			mensajeResultado = " -> NOK\n("+val1+" != "+val2+")";
			Remy.mensaje(mensaje + mensajeResultado, "fallo");
		}
		else{
			mensajeResultado = " -> OK";
			Remy.mensaje(mensaje + mensajeResultado, "correcto");
		}
		return resultado;
	},

	/**	Comprueba el valor de un campo de la pÃ¡gina.
		@param elemento Nombre del identificador del elemento cuyo valor se quiere comprobar.
		@param valor Valor de comprobaciÃ³n.
		@param	mensaje Texto que se escribirÃ¡ en la salida del test.
	**/
	probarValorIgual: function(elemento, valor, mensaje){
		return this.probarIgual($('iframe').contentDocument.getElementById(elemento).value, valor, mensaje);
	},
	probarValorListaIgual: function(elemento, valor, mensaje){
		lista = $('iframe').contentDocument.getElementById(elemento);
		valorOpcion = lista.options[lista.selectedIndex].value;
		return this.probarIgual(valorOpcion, valor, mensaje);
	},
	probarValorInnerHTMLIgual: function(elemento, valor, mensaje){
		texto = $('iframe').contentDocument.getElementById(elemento).innerHTML;
		return this.probarIgual(texto, valor, mensaje);
	},

	/**	Pone el foco en un elemento de la pÃ¡gina.
		@param elemento Nombre del identificador del elemento cuyo valor se quiere comprobar.
	**/
	enfocar: function(elemento){
		$('iframe').contentDocument.getElementById(elemento).focus();
	},

	/**	Quita el foco en un elemento de la pÃ¡gina.
		@param elemento Nombre del identificador del elemento cuyo valor se quiere comprobar.
	**/
	desenfocar: function(elemento){
		$('iframe').contentDocument.getElementById(elemento).blur();
	},

	/**	Comprueba el valor md5 de la pÃ¡gina actual.
		@param md5 Valor md5 a comprobar.
		@param	mensaje Texto que se escribirÃ¡ en la salida del test.
	**/
	probarMD5Pagina: function(md5, mensaje){
		return Remy.probarIgual(Remy.verPaginaMD5(), md5, mensaje);
	},

	md5: function(texto){
	/**-	Devuelve la codificaciÃ³n md5 de un texto.
		@param texto Texto para codificar.
	**/
		return hex_md5(texto);
	},

	/**	Espera el nÃºmero de segundos indicado.
		La espera se realiza deteniendo la ejecuciÃ³n durante el tiempo indicado.
		@param segundos Tiempo de la espera.
	**/
	esperar: function(segundos){
		if (arguments.length == 0) segundos = 2;	//Tiempo de espera por defecto.
		Remy.pararEjecucion = true;	//Para que no siga ejecutando y espere...
		window.setTimeout('Remy.ejecutar()', segundos*1000);
	},

	/**	Espera la carga del iframe y continua la ejecuciÃ³n del test o, si no estÃ¡ cargado, lo parsea.
	**/
	_esperarCarga: function(){
		if(!Remy.iframeCargado){
			Remy.pararEjecucion = true;	//Para que no siga ejecutando y espere...
			Remy.debug("Esperando la carga...");
			window.setTimeout('Remy._esperarCarga()', 2*1000);
		}
		else{
			Remy.debug("Â¡Cargado!");
			Remy.debug("Reanudando ejecuciÃ³n.");
			Remy.ejecutar();
		}
	},

	esperarCarga: function(){
		Remy.iframeCargado = false;
		Remy._esperarCarga();
		return null;
	},

	ponerValor: function(elemento, valor){
		$('iframe').contentDocument.getElementById(elemento).value = valor;
		//$('iframe').contentDocument.getElementById(elemento).setAttribute("value",valor);
		return null;	//No podemos saber si lo ha hecho bien.
	},

	dispararEvento: function(evento, elemento){
		//CÃ³digo de http://www.rakshith.net/blog/?p=35
		// Para IE 
		// Element.fireEvent(eventName)
		// txt.fireEvent(Â¿onChangeÂ¿);
		//
		var evt = $('iframe').contentDocument.createEvent('HTMLEvents');
		evt.initEvent(evento, true, true);	// evento, buble, default action.
		$('iframe').contentDocument.getElementById(elemento).dispatchEvent(evt);
	},

	///	Reinicia la ejecuciÃ³n del test cargado borrando previamente los resultados
	reiniciar: function(){
		Remy.indice = 0;
		Remy.borrarResultados;
		Remy.mostrarTest();
		Remy.ejecutar();
	   },

	///	Borra la informaciÃ³n de resultados
	borrarResultados: function (){
		while ($('divResultados').hasChildNodes()) 
			$('divResultados').removeChild($('divResultados').firstChild);
	},

	marcar: function(idRadio){
	/**-	Marca el radio button indicado.
		@param idRadio Identificador del radio que se quiere seleccionar.
	**/
		var radio = $('iframe').contentDocument.getElementById(idRadio);
		if (!radio) return
		else radio.setAttribute("checked", "checked");
		return null;
	},
	pulsar: function(idBoton){
	/**-	Lanza el evento "click" al objeto indicado.
		@param idBoton Identificador del objeto al que lanzar el evento.
	**/
		var boton = $('iframe').contentDocument.getElementById(idBoton);
		if (!boton) return
		else boton.click();
		return null;
	}
	//No olivdes quitar la coma del Ãºltimo elemento de la enumeraciÃ³n.
};

//Remy.cargarJS("js/md5.js");		//Cargamos la librerÃ­a para utilizar md5
